@@ -1,14 +1,16 @@
 import json
 import logging
+from typing import Optional
+
 import jsonpickle
-from src.entities.price import Price
-from src.entities.order import Queue, Order
-from src.entities.pedido import Fila
-from src.entities.pedido import Pedido
-import src
-from src.entities.response import CatalogoResponse, PriceResponse
 import requests
-from src.services import slack
+
+import src
+from src.entities.invoice import Invoice
+from src.entities.order import Queue, Order
+from src.entities.price import Price
+from src.entities.response import CatalogoResponse, PriceResponse, InvoiceResponse
+from src.entities.tracking import Tracking
 
 logger = logging.getLogger()
 
@@ -117,7 +119,8 @@ def get_order(url: str, token: str, order_id: int) -> Order:
         else:
             logger.warning(f'Retorno sem sucesso {response.status_code} - {response.url}')
     except Exception as ex:
-        logger.error(f'Erro ao realizar GET na api okvendas {url}' + str(ex), exc_info=True)
+        logger.error(f'Erro ao realizar GET na api okvendas {url} - {str(ex)}')
+        raise
 
     return order
 
@@ -161,3 +164,61 @@ def put_protocol_orders(order_id) -> bool:
     except Exception as ex:
         logger.error(f'Erro ao protocolar pedidos na api okvendas {url}' + str(ex), exc_info=True)
         return False
+
+
+def post_invoices(url: str, invoice: Invoice, token) -> Optional[InvoiceResponse]:
+    """
+    Enviar NF de um pedido para api okvendas
+    Args:
+        url: Url da api okvendas
+        invoice: Objeto com os dados da NF
+        token: Token de acesso da api okvendas
+
+    Returns:
+    None se o envio for sucesso. Caso falhe, um objeto contendo status e descrição do erro
+    """
+    try:
+        headers = {'Content-type': 'application/json',
+                   'Accept': 'application/json',
+                   'access-token': token}
+
+        json_invoice = jsonpickle.encode(invoice, unpicklable=False)
+        logger.info(f'{url} - {json_invoice}')
+        response = requests.post(url, json=json.loads(json_invoice), headers=headers)
+        if response.ok:
+            return None
+        else:
+            err = jsonpickle.decode(str(response.text))
+            return InvoiceResponse(**err)
+
+    except Exception as ex:
+        return InvoiceResponse(3, str(ex))
+
+
+def post_tracking(url: str, tracking: Tracking, token) -> Optional[InvoiceResponse]:
+    """
+    Enviar rastreio de um pedido para api okvendas
+    Args:
+        url: Url da api okvendas
+        tracking: Objeto com os dados do rastreio
+        token: Token de acesso da api okvendas
+
+    Returns:
+    None se o envio for sucesso. Caso falhe, um objeto contendo status e descrição do erro
+    """
+    try:
+        headers = {'Content-type': 'application/json',
+                   'Accept': 'application/json',
+                   'access-token': token}
+
+        json_tracking = jsonpickle.encode(tracking, unpicklable=False)
+        logger.info(f'{url} - {json_tracking}')
+        response = requests.post(url, json=json.loads(json_tracking), headers=headers)
+        if response.ok:
+            return None
+        else:
+            err = jsonpickle.decode(str(response.text))
+            return TrackingResponse(**err)
+
+    except Exception as ex:
+        return TrackingResponse(3, str(ex))
