@@ -15,3 +15,15 @@ class PriceResponse:
         self.status = Status
         self.message = Message
         self.protocolo = protocolo
+
+
+class InvoiceResponse:
+    def __init__(self, Status: int, Message: str):
+        self.status = Status
+        self.message = Message
+
+
+class TrackingResponse:
+    def __init__(self, Status: int, Message: str):
+        self.status = Status
+        self.message = Message
\ No newline at end of file
