@@ -1,14 +1,16 @@
 import logging
 from datetime import datetime
+from time import sleep
 
-from src.entities.order import Order
-from src.entities.pedido import Pedido
 import src
+import src.api.okvendas as api_okvendas
 import src.database.connection as database
+import src.database.utils as utils
 from src.database import queries
 from src.database.utils import DatabaseConfig
-import src.database.utils as utils
-import src.api.okvendas as api_okvendas
+from src.entities.tracking import Tracking
+from src.entities.invoice import Invoice
+from src.entities.order import Order
 
 logger = logging.getLogger()
 
@@ -34,52 +36,106 @@ def define_job_start(job_config: dict) -> None:
 
 
 def job_orders(job_config: dict, start_at_pending: bool = False) -> None:
-    db_config = utils.get_database_config(job_config)
-    if start_at_pending:
-        process_order_queue(queue_status.get('pending'), db_config)
+    try:
+        db_config = utils.get_database_config(job_config)
+        if start_at_pending:
+            process_order_queue(queue_status.get('pending'), db_config)
 
-    process_order_queue(queue_status.get('paid'), db_config)
+        process_order_queue(queue_status.get('paid'), db_config)
 
-    process_order_queue(queue_status.get('canceled'), db_config)
+        process_order_queue(queue_status.get('canceled'), db_config)
 
-    process_order_queue(queue_status.get('invoiced'), db_config)
+        # process_order_queue(queue_status.get('invoiced'), db_config)
 
-    process_order_queue(queue_status.get('shipped'), db_config)
+        process_order_queue(queue_status.get('shipped'), db_config)
 
-    process_order_queue(queue_status.get('delivered'), db_config)
+        process_order_queue(queue_status.get('delivered'), db_config)
+
+    except Exception as e:
+        logger.error(current_job + f' | Erro ao inicializar job: {str(e)}')
+
+
+def job_invoice_orders(job_config: dict):
+    # TODO Mudar para lógica de utilizar query com join no banco do cliente para consultar NFs
+    #   Add coluna na tabela pedido do semaforo para a data de sincronizacao da NF
+    #   Fazer o mesmo com o rastreio
+    try:
+        db_config = utils.get_database_config(job_config)
+        if db_config.sql is None:
+            logger.warning(job_config.get('job_name') + ' | Comando sql para baixar notas fiscais nao encontrado')
+        else:
+            invoices = query_invoices(db_config)
+            for invoice in invoices:
+                invoice_sent = api_okvendas.post_invoices(src.client_data.get('url_api') + '/pedido/faturar', invoice, src.client_data.get('token_api'))
+                if invoice_sent is None:
+                    logger.info(job_config.get('job_name') + f' | NF do pedido {invoice.id} enviada com sucesso para api okvendas')
+                    continue
+                else:
+                    logger.error(job_config.get('job_name') + f' | Falha ao enviar NF do pedido {invoice.id} para api okvendas: {invoice_sent.message}')
+    except Exception as e:
+        logger.error(job_config.get('job_name') + f' | Falha na execução do job: {str(e)}')
+
+
+def job_send_erp_tracking(job_config: dict):
+    try:
+        db_config = utils.get_database_config(job_config)
+        if db_config.sql is None:
+            logger.warning(job_config.get('job_name') + ' | Comando sql para consultar rastreios nao encontrado')
+        else:
+            trackings = query_trackings(db_config)
+            for tracking in trackings:
+                tracking_sent = api_okvendas.post_tracking(src.client_data.get('url_api') + '/pedido/encaminhar', tracking, src.client_data.get('token_api'))
+                if tracking_sent is None:
+                    logger.info(job_config.get('job_name') + f' | Rastreio do pedido {tracking.id} enviada com sucesso para api okvendas')
+                    continue
+                else:
+                    logger.error(job_config.get('job_name') + f' | Falha ao enviar rastreio do pedido {tracking.id} para api okvendas: {tracking_sent.message}')
+    except Exception as e:
+        logger.error(job_config.get('job_name') + f' | Falha na execução do job: {str(e)}')
 
 
 def process_order_queue(status: str, db_config: DatabaseConfig) -> None:
-    queue = api_okvendas.get_order_queue(
-        url=src.client_data.get('url_api') + '/pedido/fila/{0}',
-        token=src.client_data.get('token_api'),
-        status=status,
-        limit=default_limit)
-
-    for q_order in queue:
-        logger.info(current_job + f' | Iniciando processamento do pedido {q_order.order_id}')
-        order = api_okvendas.get_order(
-            url=src.client_data.get('url_api') + '/pedido/{0}',
+    try:
+        queue = api_okvendas.get_order_queue(
+            url=src.client_data.get('url_api') + '/pedido/fila/{0}',
             token=src.client_data.get('token_api'),
-            order_id=q_order.order_id)
+            status=status,
+            limit=default_limit)
 
-        if order.erp_code is not None and order.erp_code != '':
-            if check_order_existence(db_config, order.order_id):
-                logger.info(current_job + ' | Pedido ja integrado com o ERP, chamando procedure de atualizacao...')
-                if call_update_order_procedure(db_config, order):
-                    logger.info(current_job + ' | Pedido atualizado com sucesso')
-                    protocol_order(db_config, order)
-            else:
-                logger.warning(current_job + ' | ')
-        else:
-            logger.info(current_job + ' | Inserindo pedido no banco semaforo')
-            inserted = insert_temp_order(order, db_config)
-            if inserted:
-                logger.info(current_job + ' | Pedido inserido com sucesso, chamando procedures...')
-                sp_success, client_erp_id, order_erp_id = call_order_procedures(db_config, q_order.order_id)
-                if sp_success:
-                    logger.info(current_job + ' | Chamadas das procedures executadas com sucesso, protocolando pedido...')
-                    protocol_order(db_config, order, order_erp_id, client_erp_id)
+        qty = 0
+        for q_order in queue:
+            try:
+                sleep(0.5)
+                qty = qty + 1
+                logger.info(current_job + f' | Iniciando processamento ({qty} de {len(queue)}) pedido {q_order.order_id}')
+                order = api_okvendas.get_order(
+                    url=src.client_data.get('url_api') + '/pedido/{0}',
+                    token=src.client_data.get('token_api'),
+                    order_id=q_order.order_id)
+
+                if order.erp_code is not None and order.erp_code != '':
+                    if check_order_existence(db_config, order.order_id):
+                        logger.info(current_job + ' | Pedido ja integrado com o ERP, chamando procedure de atualizacao...')
+                        if call_update_order_procedure(db_config, order):
+                            logger.info(current_job + ' | Pedido atualizado com sucesso')
+                            protocol_order(db_config, order)
+                    else:
+                        logger.warning(current_job + f' | Pedido {order.order_id} nao existe no banco semaforo porem ja foi integrado previamente. Protocolando pedido...')
+                        protocol_non_existent_order(order)
+
+                else:
+                    logger.info(current_job + ' | Inserindo pedido no banco semaforo')
+                    inserted = insert_temp_order(order, db_config)
+                    if inserted:
+                        logger.info(current_job + ' | Pedido inserido com sucesso, chamando procedures...')
+                        sp_success, client_erp_id, order_erp_id = call_order_procedures(db_config, q_order.order_id)
+                        if sp_success:
+                            logger.info(current_job + ' | Chamadas das procedures executadas com sucesso, protocolando pedido...')
+                            protocol_order(db_config, order, order_erp_id, client_erp_id)
+            except Exception as e:
+                logger.error(current_job + f' | Erro no processamento do pedido {q_order.order_id}: {str(e)}')
+    except Exception as e:
+        logger.error(current_job + f' | Erro ao inicializar job de processamento de pedidos: {str(e)}')
 
 
 def protocol_order(db_config: DatabaseConfig, order: Order, order_erp_id: str = '', client_erp_id: str = '') -> None:
@@ -124,6 +180,17 @@ def protocol_order(db_config: DatabaseConfig, order: Order, order_erp_id: str =
         logger.error(f'Erro ao protocolar pedidos: {e}')
 
 
+def protocol_non_existent_order(order: Order) -> None:
+    try:
+        protocoled_order = api_okvendas.put_protocol_orders(order.order_id)
+        if protocoled_order:
+            logger.info(current_job + f' | Pedido {order.order_id} protocolado via api OkVendas')
+        else:
+            logger.warning(current_job + f' | Falha ao protocolar pedido {order.order_id} via api OkVendas')
+    except Exception as ex:
+        logger.error(f'Erro ao protocolar pedido {order.order_id}: {str(ex)}')
+
+
 def insert_temp_order(order: Order, db_config: DatabaseConfig) -> bool:
     step = ''
     db = database.Connection(db_config)
@@ -134,7 +201,7 @@ def insert_temp_order(order: Order, db_config: DatabaseConfig) -> bool:
 
         # insere cliente
         step = 'insere cliente'
-
+        logger.info(f'\tPedido {order.order_id}: Inserindo cliente')
         cursor.execute(queries.get_insert_client_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
             order.user.name,
             order.user.company_name,
@@ -166,6 +233,7 @@ def insert_temp_order(order: Order, db_config: DatabaseConfig) -> bool:
 
         # insere pedido
         step = 'insere pedido'
+        logger.info(f'\tPedido {order.order_id}: Inserindo cabecalho pedido')
         cursor.execute(queries.get_insert_order_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
             order.order_id,
             order.order_code,
@@ -199,6 +267,7 @@ def insert_temp_order(order: Order, db_config: DatabaseConfig) -> bool:
 
         # insere itens
         step = 'insere itens'
+        logger.info(f'\tPedido {order.order_id}: Inserindo itens do pedido')
         for item in order.items:
             cursor.execute(queries.get_insert_order_items_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
                 order.order_id,
@@ -215,7 +284,7 @@ def insert_temp_order(order: Order, db_config: DatabaseConfig) -> bool:
         conn.close()
         return True
     except Exception as e:
-        logger.error(current_job + f' | Passo {step} - Erro durante a inserção dos dados do pedido {order.order_id}: {str(e)}', exc_info=True)
+        logger.error(current_job + f' | Passo {step} - Erro durante a inserção dos dados do pedido {order.order_id}: {str(e)}')
         conn.rollback()
         conn.close()
         return False
@@ -255,7 +324,7 @@ def call_order_procedures(db_config: DatabaseConfig, order_id: int) -> (bool, st
         conn.close()
         return success, client_erp_id, order_erp_id
     except Exception as e:
-        logger.error(current_job + f' | Erro: {e}', exc_info=True)
+        logger.error(current_job + f' | Erro: {e}')
         conn.rollback()
         conn.close()
 
@@ -281,7 +350,7 @@ def call_update_order_procedure(db_config: DatabaseConfig, order: Order) -> bool
         conn.close()
         return success
     except Exception as e:
-        logger.error(current_job + f' | Erro: {e}', exc_info=True)
+        logger.error(current_job + f' | Erro: {e}')
         conn.rollback()
         conn.close()
 
@@ -299,3 +368,61 @@ def check_order_existence(db_config: DatabaseConfig, order_id: int) -> bool:
     except Exception as e:
         conn.close()
         return False
+
+
+def query_invoices(db_config: DatabaseConfig) -> list[Invoice]:
+    """
+    Consulta as notas fiscais a serem enviadas na api do okvendas
+    Args:
+        db_config: Configuracao do banco de dados
+
+    Returns:
+        Lista de notas fiscais para enviar
+    """
+    db = database.Connection(db_config)
+    conn = db.get_conect()
+    cursor = conn.cursor()
+    invoices = list[Invoice]
+    try:
+        cursor.execute(db_config.sql.replace(';', ''))
+        rows = cursor.fetchall()
+        columns = [col[0].lower() for col in cursor.description]
+        results = [dict(zip(columns, row)) for row in rows]
+        cursor.close()
+        conn.close()
+        if len(results) > 0:
+            invoices = [Invoice(**p) for p in results]
+
+    except Exception as ex:
+        logger.error(f' Erro ao consultar notas fiscais no banco:  {ex}')
+
+    return invoices
+
+
+def query_trackings(db_config: DatabaseConfig) -> list[Tracking]:
+    """
+    Consulta os rastreios a serem enviados na api do okvendas
+    Args:
+        db_config: Configuracao do banco de dados
+
+    Returns:
+        Lista de notas fiscais para enviar
+    """
+    db = database.Connection(db_config)
+    conn = db.get_conect()
+    cursor = conn.cursor()
+    trackings = list[Tracking]
+    try:
+        cursor.execute(db_config.sql.replace(';', ''))
+        rows = cursor.fetchall()
+        columns = [col[0].lower() for col in cursor.description]
+        results = [dict(zip(columns, row)) for row in rows]
+        cursor.close()
+        conn.close()
+        if len(results) > 0:
+            trackings = [Tracking(**p) for p in results]
+
+    except Exception as ex:
+        logger.error(f' Erro ao consultar rastreios no banco: {ex}')
+
+    return trackings
